from django.contrib import admin

from django_secret_sharing import models


@admin.register(models.Secret)
class SecretAdmin(admin.ModelAdmin):
    list_display = [
        "id",
        "erased",
        "view_once",
        "expires_at",
        "erased_at",
        "created_at",
    ]
    list_filter = [
        "erased",
        "view_once",
        "expires_at",
        "created_at",
        "erased_at",
    ]



# Generated by Django 3.0.12 on 2021-03-15 09:49
import uuid

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Secret",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("value", models.BinaryField(blank=True, null=True)),
                ("erased", models.BooleanField(default=False)),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        default=django.utils.timezone.now,
                        verbose_name="created at",
                    ),
                ),
                (
                    "modified_at",
                    models.DateTimeField(auto_now=True, verbose_name="modified at"),
                ),
                ("erased_at", models.DateTimeField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "Secret",
                "verbose_name_plural": "Secrets",
            },
        ),
    ]



from django.http import Http404
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils.decorators import method_decorator
from django.views import generic
from django.views.decorators.cache import never_cache
from django.views.decorators.debug import sensitive_post_parameters

from django_secret_sharing import settings
from django_secret_sharing.exceptions import SecretNotFound
from django_secret_sharing.forms import CreateSecretForm
from django_secret_sharing.utils import create_secret, get_secret_by_url_part


@method_decorator(never_cache, name="dispatch")
@method_decorator(sensitive_post_parameters("value"), name="dispatch")
class CreateSecretView(generic.FormView):
    template_name = "django_secret_sharing/create.html"
    form_class = CreateSecretForm

    def get_success_url(self):
        return reverse("django_secret_sharing:create")

    def form_valid(self, form):
        secret, url_part = create_secret(
            value=form.cleaned_data.get("value"),
            expires_in=form.cleaned_data.get("expires"),
            view_once=form.cleaned_data.get("view_once"),
        )

        context = super().get_context_data()
        context["secret"] = secret
        context["secret_url"] = self.request.build_absolute_uri(
            reverse("django_secret_sharing:retrieve", kwargs={"url_part": url_part})
        )
        return self.render_to_response(context)


@method_decorator(never_cache, name="dispatch")
class RetreiveSecretView(generic.TemplateView):
    template_name = "django_secret_sharing/retrieve.html"

    def get_context_data(self, **kwargs):
        url_part = kwargs["url_part"]

        try:
            secret, value = get_secret_by_url_part(url_part)
        except SecretNotFound:
            raise Http404()

        context = super().get_context_data(**kwargs)
        context["secret"] = secret
        context["url_part"] = url_part
        return context


@method_decorator(never_cache, name="dispatch")
class ViewSecretView(generic.TemplateView):
    template_name = "django_secret_sharing/view.html"

    def get_context_data(self, **kwargs):
        url_part = kwargs["url_part"]

        try:
            secret, value = get_secret_by_url_part(url_part)
        except SecretNotFound:
            raise Http404()

        if secret.view_once:
            secret.erase()

        context = super().get_context_data(**kwargs)
        context["secret"] = secret
        context["value"] = value
        return context


class GeneratePasswordView(CreateSecretView):
    def get_initial(self):
        initial = super().get_initial()
        initial["value"] = get_random_string(settings.PASSWORD_LENGTH)
        return initial



from django.urls import path

from django_secret_sharing import views

app_name = "django_secret_sharing"


urlpatterns = [
    path("", views.CreateSecretView.as_view(), name="create"),
    path(
        "generate-password/",
        views.GeneratePasswordView.as_view(),
        name="generate-password",
    ),
    path("<str:url_part>/", views.RetreiveSecretView.as_view(), name="retrieve"),
    path("<str:url_part>/view/", views.ViewSecretView.as_view(), name="view"),
]



from django import forms
from django.utils.translation import gettext_lazy as _

from django_secret_sharing import settings


class CreateSecretForm(forms.Form):
    expires = forms.TypedChoiceField(
        label=_("Expires in"), choices=settings.EXPIRY_TIME_CHOICES, coerce=int
    )
    view_once = forms.BooleanField(
        label=_("View once"),
        widget=forms.CheckboxInput(attrs={"checked": "checked"}),
        required=False,
    )
    value = forms.CharField(
        widget=forms.Textarea(attrs={"placeholder": _("Secret content here")})
    )



from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from django_secret_sharing import settings


class SecretCreateSerializer(serializers.Serializer):
    value = serializers.CharField(required=True)
    view_once = serializers.BooleanField(required=False, default=True)
    expires = serializers.ChoiceField(choices=settings.EXPIRY_TIME_CHOICES)


class SecretRetrieveSerializer(serializers.Serializer):
    url_part = serializers.CharField(required=True)



from datetime import datetime, timedelta
from typing import Optional, Tuple, Union

from Crypto.Cipher import AES
from django.core import signing
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode

from django_secret_sharing import settings
from django_secret_sharing.exceptions import SecretNotFound
from django_secret_sharing.models import Secret

URL_PART_ENCODING = "utf-8"


def encrypt_value(value, key, iv):
    cipher = AES.new(key, AES.MODE_CFB, iv)
    return cipher.encrypt(value)


def decrypt_value(value, key, iv):
    cipher = AES.new(key, AES.MODE_CFB, iv)
    return cipher.decrypt(value)


def build_url_part(signed_id, key, iv):
    return urlsafe_base64_encode(f"{signed_id}{key}{iv}".encode(URL_PART_ENCODING))


def create_secret(
    value: str, expires_in: Optional[int] = None, view_once: Optional[bool] = True
) -> Tuple[Secret, str]:
    key = get_random_string(32)
    iv = get_random_string(16)
    expiry_date = get_date_by_expires_value(expires_in)
    encrypted_value = encrypt_value(value, key=key, iv=iv)
    secret = Secret.objects.create(
        value=encrypted_value, expires_at=expiry_date, view_once=view_once
    )
    signed_id = signing.dumps(str(secret.id), salt=key)
    url_part = build_url_part(signed_id, key, iv)
    return secret, url_part


def get_secret_by_url_part(url_part) -> Tuple[Secret, str]:
    try:
        decoded_url_part = urlsafe_base64_decode(url_part).decode(URL_PART_ENCODING)
    except UnicodeDecodeError:
        raise SecretNotFound()

    iv_length = 16
    signed_id_length = len(decoded_url_part) - (32 + iv_length)

    signed_id = decoded_url_part[:signed_id_length]
    key = decoded_url_part[signed_id_length:-iv_length]
    iv = decoded_url_part[-iv_length:]

    secret_id = validate_signed_id(signed_id, salt=key)

    try:
        secret = Secret.objects.get_non_erased().get(id=secret_id)
        if secret.has_expired():
            secret.erase()
            raise Secret.DoesNotExist()
    except Secret.DoesNotExist:
        raise SecretNotFound()

    decrypted_value = decrypt_value(force_bytes(secret.value), key=key, iv=iv)
    decoded_value = decrypted_value.decode("utf-8")

    return secret, decoded_value


def validate_signed_id(signed_id, salt):
    try:
        return signing.loads(signed_id, salt=salt)
    except signing.BadSignature:
        return None


def get_date_by_expires_value(expires_value: int) -> Union[datetime, None]:
    try:
        return timezone.now() + timedelta(seconds=expires_value)
    except:
        return None



# Generated by Django 3.2.12 on 2022-05-25 10:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_secret_sharing", "0001_add_secret_model"),
    ]

    operations = [
        migrations.AddField(
            model_name="secret",
            name="expires_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="secret",
            name="view_once",
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name="secret",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]



from django.http import Http404
from django.utils.decorators import method_decorator
from django.views.decorators.cache import never_cache
from django.views.decorators.debug import sensitive_post_parameters
from rest_framework import permissions
from rest_framework.response import Response
from rest_framework.views import APIView

from django_secret_sharing import serializers
from django_secret_sharing.exceptions import SecretNotFound
from django_secret_sharing.utils import create_secret, get_secret_by_url_part


@method_decorator(never_cache, name="dispatch")
@method_decorator(sensitive_post_parameters("value"), name="dispatch")
class SecretCreateView(APIView):
    def post(self, request):
        ser = serializers.SecretCreateSerializer(data=request.data)
        ser.is_valid(raise_exception=True)

        secret, url_part = create_secret(
            value=ser.data.get("value"),
            expires_in=ser.data.get("expires"),
            view_once=ser.data.get("view_once"),
        )

        return Response({"url_part": url_part}, status=200)


@method_decorator(never_cache, name="dispatch")
@method_decorator(sensitive_post_parameters("url_part"), name="dispatch")
class SecretRetrieveView(APIView):
    permission_classes = [permissions.AllowAny]

    def post(self, request):
        ser = serializers.SecretRetrieveSerializer(data=request.data)
        ser.is_valid(raise_exception=True)

        url_part = ser.data.get("url_part")

        try:
            secret, value = get_secret_by_url_part(url_part)
        except SecretNotFound:
            raise Http404()

        if secret.view_once:
            secret.erase()

        return Response({"value": value})



from django.conf import settings
from django.utils.translation import gettext_lazy as _


def get_setting(name: str, default=None):
    return getattr(settings, "DJANGO_SECRET_SHARING_%s" % name, default)


EXPIRY_TIME_CHOICES = [
    (None, _("Select expiry time")),
    ((60 * 60), _("1 hour")),
    ((60 * 60 * 24), _("1 day")),
    ((60 * 60 * 24 * 7), _("1 week")),
]


EXPIRY_TIME_CHOICES = get_setting("EXPIRY_TIME_CHOICES", default=EXPIRY_TIME_CHOICES)
PASSWORD_LENGTH = get_setting("PASSWORD_LENGTH", default=32)



from django.urls import path

from django_secret_sharing import api_views

app_name = "django_secret_sharing_api"

urlpatterns = [
    path("", api_views.SecretCreateView.as_view(), name="create"),
    path("retrieve/", api_views.SecretRetrieveView.as_view(), name="retrieve"),
]



from setuptools import find_packages, setup

install_requires = [
    "django>=2.2.8",
    "djangorestframework>=3",
    "pycrypto>=2",
]

tests_requires = [
    "black",
    "coverage",
    "flake8",
    "isort",
    "pytest",
    "pytest-cov",
    "pytest-django",
]

setup(
    name="django-secret-sharing",
    version="0.3.0",
    description="",
    author="R. Moorman <rob@vicktor.nl>",
    install_requires=install_requires,
    tests_requires=tests_requires,
    extras_require={"test": tests_requires},
    package_dir={"": "src"},
    packages=find_packages("src"),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



import uuid

from django.conf import settings
from django.db import models
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class MutationDateModel(models.Model):
    created_at = models.DateTimeField(
        _("created at"), auto_now_add=True, editable=False
    )
    modified_at = models.DateTimeField(_("modified at"), auto_now=True, editable=False)

    class Meta:
        abstract = True


class SecretManager(models.Manager):
    def get_non_erased(self):
        return self.filter(erased=False)


class Secret(MutationDateModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    value = models.BinaryField(blank=True, null=True)
    erased = models.BooleanField(default=False)
    view_once = models.BooleanField(default=True)
    expires_at = models.DateTimeField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    erased_at = models.DateTimeField(blank=True, null=True)

    objects = SecretManager()

    def erase(self, *args, **kwargs):
        self.value = None
        self.erased = True
        self.expires_at = None
        self.erased_at = timezone.now()
        self.save()

    def has_expired(self):
        if self.erased:
            return True
        if self.expires_at:
            return timezone.now() > self.expires_at
        return False

    class Meta:
        verbose_name = _("Secret")
        verbose_name_plural = _("Secrets")
